from flask import Flask, flash, render_template, request, redirect, url_for, jsonify, abort, Response
from flask_sqlalchemy import SQLAlchemy
import sqlite3
from flask_login import UserMixin, LoginManager, login_required, login_user, logout_user, current_user
from flask_wtf import FlaskForm
from wtforms import StringField, PasswordField, SubmitField, TextAreaField
from wtforms.validators import InputRequired, Length
from wtforms.validators import Email
from flask_migrate import Migrate
from wtforms.validators import InputRequired, Length, Email 
from flask_mail import Mail
from werkzeug.security import generate_password_hash, check_password_hash
import os
from werkzeug.utils import secure_filename
import re
from flask import send_from_directory
from datetime import datetime
from werkzeug.security import generate_password_hash
app = Flask(__name__, static_url_path='/static')
basedir = os.path.abspath(os.path.dirname((__file__)))
database = "app.db"
con = sqlite3.connect(os.path.join(basedir, database))
mail = Mail(app)
app.config['SECRET_KEY'] = "jhkxhiuydu"
app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///' + os.path.join(basedir, database)
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = True
app.config['FLASK_ADMIN_SWATCH'] = 'cerulean'
app.config['MAIL_SERVER'] = 'intexcoin.com'
app.config['MAIL_PORT'] = 465
app.config['MAIL_USERNAME'] = 'info@intexcoin.com'
app.config['MAIL_SERVER'] = 'server148.web-hosting.com'
#app.config['UPLOAD_FOLDER'] = 'uploads'  # Folder for uploaded files
# app.config['UPLOAD_FOLDER'] = os.path.join(basedir, 'static/uploads')  # Folder for uploaded files
app.config['ALLOWED_EXTENSIONS'] = {'png', 'jpg', 'jpeg', 'gif'}  # Allowed file extensions
# Assuming your app variable is named 'app'
app.config['UPLOAD_FOLDER'] = 'uploads'

app.add_url_rule('/uploads/<filename>', 'uploads', build_only=True)


login_manager = LoginManager(app)
login_manager.login_view = 'login'

db = SQLAlchemy(app)
migrate = Migrate(app, db)

@login_manager.user_loader
def load_user(user_id):
    return User.query.get(int(user_id))


class User(db.Model, UserMixin):
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(50), unique=True, nullable=False)
    password = db.Column(db.String(50), nullable=False)
    email = db.Column(db.String(120), unique=True, nullable=False)
    country = db.Column(db.String(100), nullable=False)
    citizenship = db.Column(db.String(100), nullable=False)
    amount = db.Column(db.Integer, default = 0)


class Transaction(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    type = db.Column(db.String(50))  # e.g., 'Deposit', 'Purchase'
    amount = db.Column(db.Float)
    timestamp = db.Column(db.DateTime, default=datetime.utcnow)
    user_id = db.Column(db.Integer, db.ForeignKey('user.id'))

    def __repr__(self):
        return f'<Transaction {self.type} - ${self.amount}>'



class Withdraw(db.Model, UserMixin):
    id = db.Column(db.Integer, primary_key = True)
    amount = db.Column(db.String(255))
    withdraw_method = db.Column(db.String(255))
    
    

class Schedule(db.Model, UserMixin):
    id = db.Column(db.Integer, primary_key = True)
    name = db.Column(db.String(255))
    main_image = db.Column(db.LargeBinary)
    main_image_filename=db.Column(db.String(100))
    content = db.Column(db.String(100))
    

class Rooms(db.Model, UserMixin):
    id = db.Column(db.Integer, primary_key = True)
    name = db.Column(db.String(255))
    main_image = db.Column(db.LargeBinary)
    main_image_filename=db.Column(db.String(100))

class Booking(db.Model, UserMixin):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(255))
    email = db.Column(db.String(255))
    phone_number = db.Column(db.String(20))
    num_guests = db.Column(db.Integer)
    dungeon = db.Column(db.String(255))
    membership = db.Column(db.String(50))
    hours = db.Column(db.Integer)
    location = db.Column(db.String(255))
    instructions = db.Column(db.String(255))
    check_in = db.Column(db.DateTime)
    check_out = db.Column(db.DateTime)
    


 
def allowed_file(filename):
    return '.' in filename and filename.rsplit('.', 1)[1].lower() in app.config['ALLOWED_EXTENSIONS']
  
  
  
@app.route('/register', methods=['GET', 'POST'])
def register():
    if request.method == 'POST':
        username = request.form['username']
        password = request.form['password']
        email = request.form['email']
        country = request.form['country']
        citizenship = request.form['citizenship']

        # Check if the username or email is already taken
        existing_user = User.query.filter((User.username == username) | (User.email == email)).first()
        if existing_user:
            flash('Username or Email is already taken. Please choose another one.', 'danger')
            return redirect(url_for('register'))

        # Create a new user with hashed password
        hashed_password = generate_password_hash(password, method='pbkdf2:sha256')
        new_user = User(
            username=username,
            password=hashed_password,
            email=email,
            country=country,
            citizenship=citizenship
        )
        db.session.add(new_user)
        db.session.commit()

        flash('Account created successfully! You can now log in.', 'success')
        return redirect(url_for('login'))

    return render_template('register.html')
  
@app.route('/transactions')
@login_required
def transactions():
    user_id = current_user.id
    transactions = Transaction.query.filter_by(user_id=user_id).order_by(Transaction.timestamp.desc()).all()
    return render_template('transactions.html', transactions=transactions)
   

@app.route('/login', methods=['GET', 'POST'])
def login():
    if request.method == 'POST':
        username = request.form['username']
        password = request.form['password']

        # Fetch the user by username
        user = User.query.filter_by(username=username).first()

        if user and check_password_hash(user.password, password):
            login_user(user)
            flash('Login successful!', 'success')
            return redirect(url_for('dashboard'))

        flash('Login failed. Please check your username and password.', 'danger')

    return render_template('register.html')


@app.route('/logout')
@login_required
def logout():
    logout_user()
    flash('You have been logged out.', 'success')
    return redirect(url_for('login'))
@app.route('/withdraw', methods=['POST'])
@login_required
def withdraw():
    amount = float(request.form['amount'])
    withdraw_method = request.form['method']
    user = User.query.get(current_user.id)  # Assuming you use Flask-Login and User model

    if not amount or not withdraw_method:
        flash('Please fill out all fields.', 'danger')
        return redirect(url_for('dashboard'))  # Redirect to the page where the form is located

    if amount <= 0:
        flash('The withdrawal amount must be positive.', 'danger')
        return redirect(url_for('dashboard'))

    if amount > user.balance:
        flash('Insufficient funds. Please check your balance.', 'danger')
        return redirect(url_for('dashboard'))

    # Create a new withdraw entry
    new_withdraw = Withdraw(
        amount=amount,
        withdraw_method=withdraw_method
    )

    try:
        # Deduct the amount from the user's balance
        user.balance -= amount
        db.session.add(new_withdraw)
        db.session.commit()
        flash('Withdrawal request submitted successfully!', 'success')
    except Exception as e:
        db.session.rollback()
        flash('An error occurred. Please try again later.', 'danger')
        # Optionally log the exception e

    return redirect(url_for('dashboard'))  # Redirect to the page where the form is located

@app.route('/dashboard')
@login_required
def dashboard():
    return render_template('dashboard.html', user=current_user)


@app.route('/uploads/<filename>')
def uploaded_file(filename):
    return send_from_directory(app.config['UPLOAD_FOLDER'], filename)

@app.route('/wallet')
def wallet():
    
    return render_template('my_wallet.html')

@app.route('/create_dommies', methods=['GET', 'POST'])
@login_required
def create_dommies():
    if request.method == 'POST':
        name = request.form['name']
        
        # Upload Main Image for Event
        main_image = request.files['main_image']
        if main_image and allowed_file(main_image.filename):
            filename = secure_filename(main_image.filename)
            main_image.save(os.path.join(app.config['UPLOAD_FOLDER'], filename))
            main_image_filename = filename
        else:
            main_image_filename = None

        new_dommie = Dommies(name=name,  main_image_filename=main_image_filename)
        db.session.add(new_dommie)
        db.session.commit()

        flash('Dommie created successfully!', 'success')
        return redirect(url_for('dashboard'))

    return render_template('create_dommie.html')

@app.route('/dommies', methods=['GET'])
def dommies():
    dommies = Dommies.query.all()
    return render_template('dommies.html', dommies=dommies)
@app.route('/dommies/<int:dommies_id>', methods = ['GET', 'POST'])
def view_dommies(dommies_id):
    dommies = Dommies.query.get(dommies_id)
    
    if not dommies:
        abort(404)
        
    return render_template('view_dommies.html', dommies = dommies )



@app.route('/dommies/<int:dommies_id>/update', methods=['GET', 'POST'])
@login_required
def update_dommies(dommies_id):
    dommies = Dommies.query.get(dommies_id)

    if not dommies:
        abort(404)

    if request.method == 'POST':
        dommies.name = request.form['name']
       

        # Update Main Image if a new one is provided
        main_image = request.files['main_image']
        if main_image and allowed_file(main_image.filename):
            filename = secure_filename(main_image.filename)
            main_image.save(os.path.join(app.config['UPLOAD_FOLDER'], filename))
            dommies.main_image_filename = filename

        db.session.commit()
        flash('Dommies updated successfully!', 'success')

        return redirect(url_for('dommies'))

    return render_template('update_dommies.html', dommies=dommies)

@app.route('/dommies/<int:dommies_id>/delete', methods=['GET', 'POST'])
@login_required
def delete_dommies(dommies_id):
    dommies = Dommies.query.get(dommies_id)

    if not dommies:
        abort(404)

    if request.method == 'POST':
        # Delete associated image from the file system (optional)
        if dommies.main_image_filename:
            try:
                os.remove(os.path.join(app.config['UPLOAD_FOLDER'], dommies.main_image_filename))
                flash('Image file deleted successfully!', 'success')
            except FileNotFoundError:
                flash(f'Warning: Image file not found at {os.path.join(app.config["UPLOAD_FOLDER"], dommies.main_image_filename)}.', 'warning')

        # Delete the dommies from the database
        db.session.delete(dommies)
        db.session.commit()
        flash('Dommies deleted successfully!', 'success')

        return redirect(url_for('dommies'))

    return render_template('delete_dommies.html', dommies=dommies)



@app.route('/schedule_list', methods=['GET'])
def schedule_list():
    all_schedule = Schedule.query.all()
    return render_template('schedule_list.html', all_schedule=all_schedule)





@app.route('/create_schedule', methods=['GET', 'POST'])
@login_required
def create_schedule():
    if request.method == 'POST':
        name = request.form['name']
        content = request.form['content']
        
        # Upload Main Image for Event
        main_image = request.files['main_image']
        if main_image and allowed_file(main_image.filename):
            filename = secure_filename(main_image.filename)
            main_image.save(os.path.join(app.config['UPLOAD_FOLDER'], filename))
            main_image_filename = filename
        else:
            main_image_filename = None

        new_schedule = Schedule(name=name,content = content,   main_image_filename=main_image_filename)
        db.session.add(new_schedule)
        db.session.commit()

        flash('schedule created successfully!', 'success')
        return redirect(url_for('dashboard'))

    return render_template('create_schedule.html')

@app.route('/schedule', methods=['GET'])
def schedule():
    schedule = Schedule.query.all()
    return render_template('schedule.html', schedule=schedule)
@app.route('/schedule/<int:schedule_id>', methods = ['GET', 'POST'])
def view_schedule(schedule_id):
    schedule = Schedule.query.get(schedule_id)
    
    if not schedule:
        abort(404)
        
    return render_template('view_schedule.html', schedule = schedule )



@app.route('/schedule/<int:schedule_id>/update', methods=['GET', 'POST'])
@login_required
def update_schedule(schedule_id):
    schedule = Schedule.query.get(schedule_id)

    if not schedule:
        abort(404)

    if request.method == 'POST':
        schedule.name = request.form['name']
        schedule.content = request.form['content']

       

        # Update Main Image if a new one is provided
        main_image = request.files['main_image']
        if main_image and allowed_file(main_image.filename):
            filename = secure_filename(main_image.filename)
            main_image.save(os.path.join(app.config['UPLOAD_FOLDER'], filename))
            schedule.main_image_filename = filename

        db.session.commit()
        flash('Schedule updated successfully!', 'success')

        return redirect(url_for('schedule'))

    return render_template('update_schedule.html', schedule=schedule)

@app.route('/schedule/<int:schedule_id>/delete', methods=['GET', 'POST'])
@login_required
def delete_schedule(schedule_id):
    schedule = Schedule.query.get(schedule_id)

    if not schedule:
        abort(404)

    if request.method == 'POST':
        # Delete associated image from the file system (optional)
        if schedule.main_image_filename:
            try:
                os.remove(os.path.join(app.config['UPLOAD_FOLDER'], schedule.main_image_filename))
                flash('Image file deleted successfully!', 'success')
            except FileNotFoundError:
                flash(f'Warning: Image file not found at {os.path.join(app.config["UPLOAD_FOLDER"], schedule.main_image_filename)}.', 'warning')

        # Delete the dommies from the database
        db.session.delete(schedule)
        db.session.commit()
        flash('Schedule deleted successfully!', 'success')

        return redirect(url_for('schedule'))

    return render_template('delete_schedule.html', schedule=schedule)

@app.route('/room_list', methods=['GET'])
def room_list():
    all_room = Rooms.query.all()
    return render_template('room_list.html', all_room=all_room)


@app.route('/create_room', methods=['GET', 'POST'])
@login_required
def create_room():
    if request.method == 'POST':
        name = request.form['name']
        
        # Upload Main Image for Event
        main_image = request.files['main_image']
        if main_image and allowed_file(main_image.filename):
            filename = secure_filename(main_image.filename)
            main_image.save(os.path.join(app.config['UPLOAD_FOLDER'], filename))
            main_image_filename = filename
        else:
            main_image_filename = None

        new_room = Rooms(name=name,  main_image_filename=main_image_filename)
        db.session.add(new_room)
        db.session.commit()

        flash('Room created successfully!', 'success')
        return redirect(url_for('dashboard'))

    return render_template('create_room.html')

@app.route('/roomlist', methods=['GET'])
def roomlist():
    room = Rooms.query.all()
    return render_template('room-list-1.html', room=room)
@app.route('/roomlist/<int:room_id>', methods = ['GET', 'POST'])
def view_room(room_id):
    room = Rooms.query.get(room_id)
    
    if not room:
        abort(404)
        
    return render_template('view_room.html', room = room )



@app.route('/roomlist/<int:room_id>/update', methods=['GET', 'POST'])
@login_required
def update_room(room_id):
    room = Rooms.query.get(room_id)

    if not room:
        abort(404)

    if request.method == 'POST':
        room.name = request.form['name']
       

        # Update Main Image if a new one is provided
        main_image = request.files['main_image']
        if main_image and allowed_file(main_image.filename):
            filename = secure_filename(main_image.filename)
            main_image.save(os.path.join(app.config['UPLOAD_FOLDER'], filename))
            room.main_image_filename = filename

        db.session.commit()
        flash('Room updated successfully!', 'success')

        return redirect(url_for('roomlist'))

    return render_template('update_room.html', room=room)

@app.route('/roomlist/<int:room_id>/delete', methods=['GET', 'POST'])
@login_required
def delete_room(room_id):
    room = Rooms.query.get(room_id)

    if not room:
        abort(404)

    if request.method == 'POST':
        # Delete associated image from the file system (optional)
        if dommies.main_image_filename:
            try:
                os.remove(os.path.join(app.config['UPLOAD_FOLDER'], dommies.main_image_filename))
                flash('Image file deleted successfully!', 'success')
            except FileNotFoundError:
                flash(f'Warning: Image file not found at {os.path.join(app.config["UPLOAD_FOLDER"], dommies.main_image_filename)}.', 'warning')

        # Delete the dommies from the database
    
        db.session.delete(room)
        db.session.commit()
        flash('Room deleted successfully!', 'success')

        return redirect(url_for('roomlist'))

    return render_template('delete_room.html', room=room)


# Define the route for the booking form
@app.route('/book', methods=['GET', 'POST'])
def book():
    if request.method == 'POST':
        # Get form data
        name = request.form['name']
        email = request.form['email']
        phone_number = request.form['phone_number']
        num_guests = int(request.form['num_guests'])
        dungeon = request.form['dungeon']
        membership = request.form['membership']
        hours = int(request.form['hours'])
        location = request.form['location']
        instructions = request.form['instructions']
        check_in = datetime.strptime(request.form['check_in'], '%Y-%m-%dT%H:%M')
        check_out = datetime.strptime(request.form['check_out'], '%Y-%m-%dT%H:%M')

        # Create a new Booking instance
        new_booking = Booking(
            name=name,
            email=email,
            phone_number=phone_number,
            num_guests=num_guests,
            dungeon=dungeon,
            membership=membership,
            hours=hours,
            location=location,
            instructions=instructions,
            check_in=check_in,
            check_out=check_out
        )

        # Add the new booking to the database
        db.session.add(new_booking)
        db.session.commit()

        # Redirect to a success page or any other desired page
        flash("Booking successful")

    return render_template('booking_form.html')


@app.route('/bookings')
def view_bookings():
    bookings = Booking.query.all()
    return render_template('bookings.html', bookings=bookings)









@app.route('/')
def home():
    return render_template('index.html')

@app.route('/nav')
def nav():
    return render_template('nav.html')
@app.route('/header')
def header():
    return render_template('header.html')
@app.route('/footer')
def footer():
    return render_template('footer.html')
# @app.route('/roomlist')
# def roomlist():
#     return render_template('room-list-1.html')
@app.route('/equipment')
def equipment():
    return render_template('restaurant.html')

# @app.route('/dommies')
# def dommies():
#     return render_template('dommies.html')

# @app.route('/schedule')
# def schedule():
#     return render_template('schedule.html')
@app.route('/contact')
def contact():
    return render_template('contact.html')
@app.route('/about')
def about():
    return render_template('about.html')











@app.route("/db")
def database():
    db.drop_all()
    db.create_all()
    return "Hello done!!!"

if __name__ == '__main__':
    app.run(debug=True)